==> donejs-typeahead-0.3.0.tgz: package/src/donejs-typeahead.js <==
import Component from 'can/component/';
import Map from 'can/map/';
import 'can/map/define/';
import './donejs-typeahead.less!';
import template from './donejs-typeahead.stache!';

// import templateResult from './typeahead-result.stache!';
// import templateResultHandlebars from './typeahead-result2.stache!';
// import templateEmpty from './typeahead-empty.stache!';
// import 'donejs-typeahead/vendor/typeahead-bundle';
// import "twitter-typeahead-components-bundle";
// import xdomainrequest from "twitter-typeahead-components-xdomainrequest";
import Bloodhound from "twitter-typeahead-components-bundle";
import Handlebars from "twitter-typeahead-components-handlebars";


/**
 *
 * @testCase Changing the default values
 */
export const ViewModel = Map.extend({
  define: {
    selectedElementValue: {
      type: 'string',
      value: 'no value has been set'
    },
    selectedElementObject: {
      type: 'object',
      value: 'no value has been set'
    },
    // tokenOpen: {
    //   type: 'string',
    //   value: '[['
    // },
    // tokenClose: {
    //   type: 'string',
    //   value: ']]'
    // }
  },

  /**
   * transform REST API response to a typehead compatible one
   *
   * typehahead works with an array of objects but the response 
   * from the REST API will contain a data object with the results
   *
   * @testCase data is not valid
   * @testCase data.data is not valid
   * @testCase the result obtained is not in the expected format ( {data: [object1, object2]} )
   */
  restApiSearchResultToArray: function(data) {
    if (data && typeof data === 'object' && data.data && Array.isArray(data.data)) {
      return data.data;
    } else {
      return [];
    }
  },


  /**
   * @todo: remove from here this sholud be a library if needed
   *
   * get information from selectElementObject
   *
   * if the key does not exist or the value is undefined
   * it will return false
   *
   * @testCase Retrieve standard types
   * @testCase Try to retrieve non existing or invalid types
   */
  getSelectedElementByKey: function(key) {
    if (key && typeof key === 'string') {
      if (this.attr('selectedElementObject')[key] !== undefined) {
        return this.attr('selectedElementObject')[key];
      } else {
        return null;
      }
    } else {
      return null;
    }
  },

  /**
   * @todo: move to helper and see if it works ok
   *
   * helper to debug the information contained in the object
   */
  serializeSelection: function() {
    return JSON.stringify(this.attr('selectedElementObject'));
  },

  /**
   * @todo: decide where to place this
   *
   * replace special tokens
   *
   * replace double brackets with double curly brackets
   *
   * @testCase wrong params
   * @testCase no instances
   * @testCase exactly one instance
   * @testCase more than one instance
   *
   * @param distriString String the original string containing tokens
   */
  replaceSpecialTokens: function(dirtyString) {
    if (dirtyString && typeof dirtyString === 'string') {
      // replace special characters with {{ or }}
      dirtyString = dirtyString.replace(/\[\[/ig, '{{');
      dirtyString = dirtyString.replace(/\]\]/ig, '}}');
      return dirtyString;
    } else {
      return '';
    }
  }

});


var addToken = function(query, settings, wildcard) {
  var jwt = "";
  try {
    jwt = window.localStorage['feathers-jwt'];
  } catch (e) {}

  if (settings.url && settings.url.indexOf(wildcard) !== -1) {
    settings.url = settings.url.replace(wildcard, query);
  } else {}

  // should the headers be undefined, set them as an empty array
  if (settings.headers === undefined) {
    settings.headers = [];
  } else {}

  // set authorization header
  if (jwt !== undefined) {
    settings.headers.Authorization = jwt.toString(); // cast to string just in case
  } else {}
  return settings;
};

export default Component.extend({
  tag: 'donejs-typeahead',
  viewModel: ViewModel,
  template,
  events: {
    // other events: "typeahead:active", "typeahead:change", "typeahead:cursorchange", "typeahead:asynccancel", "typeahead:asyncreceive"
    "inserted": function() {
      // @todo write documentation
      // @todo move to viewmodel
      var setDefaultValue = function(key, viewModel) {
        var defaultValue = {
          remoteHost: 'localhost',
          remotePort: 3030,
          remotePath: '/twitters',
          remoteProtocol: 'http://',
          displayValue: 'name',
          searchValue: 'lastName'
        };
        if (key !== undefined) {
          if (viewModel[key] !== undefined) {
            return viewModel[key];
          } else {
            if (defaultValue[key] !== undefined) {
              return defaultValue[key];
            } else {
              return null;
            }
          }
        } else {
          return null;
        }
      };

      // @todo write documentation
      // @todo move to viewmodel
      var buildRemoteUrl = function(viewModel) {
        var remoteUrl, remoteHost, remotePort, remotePath, remoteProtocol;
        remoteHost = setDefaultValue('remoteHost', viewModel);
        remotePort = setDefaultValue('remotePort', viewModel);
        remotePath = setDefaultValue('remotePath', viewModel);
        remoteProtocol = setDefaultValue('remoteProtocol', viewModel);
        remoteUrl = remoteProtocol + remoteHost + ':' + remotePort + remotePath;
        return remoteUrl;
      };



      var engine, displayValue, remoteUrl, searchValue;
      $.support.cors = true;
      // remoteHost = '/api/twitterAuths';

      remoteUrl = buildRemoteUrl(this.viewModel);
      displayValue = setDefaultValue('displayValue', this.viewModel);
      searchValue = setDefaultValue('searchValue', this.viewModel);

      // now that we have the displayValue set:
      var suggestionTemplate;
      if (this.viewModel.suggestionTemplate !== undefined) {
        suggestionTemplate = this.viewModel.suggestionTemplate;
        suggestionTemplate = this.viewModel.replaceSpecialTokens(suggestionTemplate);
      } else {
        // suggestionTemplate = '<div class="ProfileCard u-cf Typeahead-suggestion Typeahead-selectable"><div class="ProfileCard-details"><div class="ProfileCard-realName"><span style="color:green;">{{name}}</span> / {{' + displayValue + '}}</div></div></div>';
        suggestionTemplate = '<div class="donejs-typeahead-suggestion donejs-typeahead-selectable"><span class="test-1">{{name}}</span> / {{' + displayValue + '}}</div>';
      }

      var wildcard = '%QUERY';
      var datumTokenizer = Bloodhound.tokenizers.obj.whitespace(displayValue);

      engine = new Bloodhound({
        // identify: function(o) {
        //   return o.lastName;
        // },
        queryTokenizer: Bloodhound.tokenizers.whitespace,
        datumTokenizer: datumTokenizer,
        // dupDetector: function(a, b) {
        //   return a.lastName === b.lastName;
        // },
        // prefetch: remoteUrl + '',
        remote: {

          // todo verificar si aca le mando varios parametros o uno solo
          // creo que lo logico es que el servidor lo maneje como q, query o typeahead
          url: remoteUrl + '?' + searchValue + '=' + wildcard,
          wildcard,
          cache: false,
          rateLimitWait: 300,
          transform: this.viewModel.restApiSearchResultToArray,
          // rateLimitBy: 'throttle',
          prepare: function(query, settings) {
            return addToken(query, settings, wildcard);
          }
        }

      });

      function engineWithDefaults(q, sync, async) {
        engine.search(q, sync, async);
      }

      var typeaheadMainConfiguration = {
        hint: $('.Typeahead-hint'),
        menu: $('.Typeahead-menu'),
        highlight: true,
        minLength: 1,
        classNames: {
          open: 'is-open',
          empty: 'is-empty',
          cursor: 'is-active',
          suggestion: 'Typeahead-suggestion',
          selectable: 'Typeahead-selectable'
        }
      };
      var typeaheadDataConfiguration = {
        source: engineWithDefaults,
        displayKey: displayValue, // when an item is selected, this attribute will be shown in the input field
        templates: {
          // suggestion: template,
          // suggestion: templateResult,
          // suggestion: Handlebars.compile($("#result-template")),
          // suggestion: Handlebars.compile($("#result-template").html()),
          // suggestion: Handlebars.compile(templateResult),
          // suggestion: Handlebars.compile("<div>Name:{{name}} / Lastname:{{lastName}}</div>"),
          suggestion: Handlebars.compile(suggestionTemplate),
          empty: Handlebars.compile('<div class="EmptyMessage">No results here...</div>')
            // empty: templateEmpty
        }
      };

      var self = this;
      $(document).ready(function() {
        /*
         * try / catch is needed because $(self.element) is not available
         * duriong ssr and it might not be available even when document is ready
         * causing find to trigger an error
         */
        try {
          $(self.element)
            .find('input')
            .typeahead(typeaheadMainConfiguration, typeaheadDataConfiguration)
            .on('typeahead:select', function(event, data) {
              // set element data
              // 1. the object
              self.viewModel.attr('selectedElementObject', data);
              // 2. the text to show
              self.viewModel.attr('selectedElementValue', data[displayValue]);

            });
        } catch (e) {
          console.log('catch:', e.message);
        }
      });
    },
  }
});

==> donejs-typeahead-0.6.0.tgz: package/src/donejs-typeahead.js <==
import Component from 'can/component/';
import Map from 'can/map/';
import 'can/map/define/';
import './donejs-typeahead.less!';
import template from './donejs-typeahead.stache!';
// import xdomainrequest from "twitter-typeahead-components-xdomainrequest";
import Handlebars from "twitter-typeahead-components-handlebars";
import $ from 'jquery'; // available through Bloodhound indirectly
import 'twitter-typeahead-components-xdomainrequest';
import Bloodhound from "twitter-typeahead-components-bundle";


// import templateResult from './donejs-typeahead-result.stache!';
// import templateResultHandlebars from './typeahead-result2.stache!';
// import templateEmpty from './typeahead-empty.stache!';


/**
 *
 * @testCase Changing the default values
 */
export const ViewModel = Map.extend({
  define: {
    selectedElementValue: {
      type: 'string',
      value: ''
    },
    selectedElementObject: {
      type: 'object',
      value: ''
    }
  },


  /**
   * transform REST API response to a typehead compatible one
   *
   * typehahead works with an array of objects but the response 
   * from the REST API will contain a data object with the results
   *
   * @testCase data is not valid
   * @testCase data.data is not valid
   * @testCase the result obtained is not in the expected format ( {data: [object1, object2]} )
   */
  restApiSearchResultToArray: function(data) {
    if (data && typeof data === 'object' && data.data && Array.isArray(data.data)) {
      return data.data;
    } else {
      return [];
    }
  },


  /**
   * @todo: remove from here this sholud be a library if needed
   *
   * get information from selectElementObject
   *
   * if the key does not exist or the value is undefined
   * it will return false
   *
   * @testCase Retrieve standard types
   * @testCase Try to retrieve non existing or invalid types
   */
  getSelectedElementByKey: function(key) {
    if (key && typeof key === 'string') {
      if (this.attr('selectedElementObject')[key] !== undefined) {
        return this.attr('selectedElementObject')[key];
      } else {
        return null;
      }
    } else {
      return null;
    }
  },

  /**
   * @todo: move to helper and see if it works ok
   *
   * helper to debug the information contained in the object
   */
  serializeSelection: function() {
    return JSON.stringify(this.attr('selectedElementObject'));
  },

  /**
   * @todo: decide where to place this
   *
   * replace special tokens
   *
   * replace double brackets with double curly brackets
   *
   * @testCase wrong params
   * @testCase no instances
   * @testCase exactly one instance
   * @testCase more than one instance
   *
   * @param dirtyString String the original string containing tokens
   */
  replaceSpecialTokens: function(dirtyString) {
    if (dirtyString && typeof dirtyString === 'string') {
      // replace special characters with {{ or }}
      dirtyString = dirtyString.replace(/\[\[/ig, '{{');
      dirtyString = dirtyString.replace(/\]\]/ig, '}}');
      return dirtyString;
    } else {
      return '';
    }
  }

});


var addToken = function(query, settings, wildcard) {
  var jwt = "";
  try {
    jwt = window.localStorage['feathers-jwt'];
  } catch (e) {}

  if (settings.url && settings.url.indexOf(wildcard) !== -1) {
    settings.url = settings.url.replace(wildcard, query);
  } else {}

  // should the headers be undefined, set them as an empty array
  if (settings.headers === undefined) {
    settings.headers = [];
  } else {}

  // set authorization header
  if (jwt !== undefined) {
    settings.headers.Authorization = jwt.toString(); // cast to string just in case
  } else {}
  return settings;
};


export default Component.extend({
  tag: 'donejs-typeahead',
  viewModel: ViewModel,
  template,
  events: {
    // other events: "typeahead:active", "typeahead:change", "typeahead:cursorchange", "typeahead:asynccancel", "typeahead:asyncreceive"
    "inserted": function() {

      var self = this;
      // @todo write documentation
      // @todo move to viewmodel
      var setDefaultValue = function(key, viewModel) {
        var defaultValue = {
          remoteHost: 'localhost',
          remotePort: 3030,
          remotePath: '/twitters',
          remoteProtocol: 'http://',
          displayValue: 'name',
          searchValue: 'lastName'
        };
        if (key !== undefined) {
          if (viewModel[key] !== undefined) {
            return viewModel[key];
          } else {
            if (defaultValue[key] !== undefined) {
              return defaultValue[key];
            } else {
              return null;
            }
          }
        } else {
          return null;
        }
      };

      // @todo write documentation
      // @todo move to viewmodel
      var buildRemoteUrl = function(viewModel) {
        var remoteUrl, remoteHost, remotePort, remotePath, remoteProtocol;
        remoteHost = setDefaultValue('remoteHost', viewModel);
        remotePort = setDefaultValue('remotePort', viewModel);
        remotePath = setDefaultValue('remotePath', viewModel);
        remoteProtocol = setDefaultValue('remoteProtocol', viewModel);
        remoteUrl = remoteProtocol + remoteHost + ':' + remotePort + remotePath;
        return remoteUrl;
      };



      var engine, displayValue, remoteUrl, searchValue;
      $.support.cors = true;
      // remoteHost = '/api/twitterAuths';

      remoteUrl = buildRemoteUrl(this.viewModel);
      displayValue = setDefaultValue('displayValue', this.viewModel);
      searchValue = setDefaultValue('searchValue', this.viewModel);

      // now that we have the displayValue set:
      var suggestionTemplate;
      if (this.viewModel.suggestionTemplate !== undefined) {
        suggestionTemplate = this.viewModel.suggestionTemplate;
        suggestionTemplate = this.viewModel.replaceSpecialTokens(suggestionTemplate);
      } else {
        suggestionTemplate = '<div class="donejs-typeahead-suggestion donejs-typeahead-selectable"><span class="test-1">{{name}}</span> / {{' + displayValue + '}}</div>';
      }

      var wildcard = '%QUERY';
      var datumTokenizer = Bloodhound.tokenizers.obj.whitespace(displayValue);

      engine = new Bloodhound({
        // identify: function(o) {
        //   return o.lastName;
        // },
        queryTokenizer: Bloodhound.tokenizers.whitespace,
        datumTokenizer: datumTokenizer,
        // dupDetector: function(a, b) {
        //   return a.lastName === b.lastName;
        // },
        // prefetch: remoteUrl + '',
        remote: {
          // todo verificar si aca le mando varios parametros o uno solo
          // creo que lo logico es que el servidor lo maneje como q, query o typeahead
          url: remoteUrl + '?' + searchValue + '=' + wildcard,
          wildcard,
          cache: false,
          rateLimitWait: 300,
          transform: this.viewModel.restApiSearchResultToArray,
          // rateLimitBy: 'throttle',
          prepare: function(query, settings) {
            return addToken(query, settings, wildcard);
          }
        }
      });

      function engineWithDefaults(q, sync, async) {
        engine.search(q, sync, async);
      }

      var typeaheadMainConfiguration = {
        hint: $('.Typeahead-hint'),
        menu: $('.Typeahead-menu'),
        highlight: true,
        minLength: 1,
        classNames: {
          open: 'is-open',
          empty: 'is-empty',
          cursor: 'is-active',
          suggestion: 'Typeahead-suggestion',
          selectable: 'Typeahead-selectable'
        }
      };
      var typeaheadDataConfiguration = {
        source: engineWithDefaults,
        displayKey: displayValue, // when an item is selected, this attribute will be shown in the input field
        templates: {
          // suggestion: template,
          // suggestion: templateResult,
          suggestion: Handlebars.compile(suggestionTemplate),
          empty: Handlebars.compile('<div class="EmptyMessage">No results here...</div>')
            // empty: templateEmpty
        }
      };

      $(document).ready(function() {
        /*
         * try / catch is needed because $(self.element) is not available
         * during ssr and it might not be available even when document is ready
         * causing find to trigger an error
         */
        try {
          $(self.element)
          .find('input')
          .typeahead(typeaheadMainConfiguration, typeaheadDataConfiguration)
          .on('typeahead:select', function(event, data) {
            // set element data
            // 1. the object
            self.viewModel.attr('selectedElementObject', data);
            // 2. the text to show
            self.viewModel.attr('selectedElementValue', data[displayValue]);
          });
        } catch (e) {
          console.log('WARNING (donejs-typehead)', e.message);
          // console.log('catch:', e.message, self.element);
        }
      });
    },
  }
});
